import Flutter
import UIKit
import HyperKYC

public class SwiftHyperkycFlutterPlugin: NSObject, FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        
        let hyperKycChannel = FlutterMethodChannel(name: FLHyperKycConstants.HYPERKYC_CHANNEL_NAME, binaryMessenger: registrar.messenger())
        
        hyperKycChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result:@escaping FlutterResult) -> Void in
            if (call.method.caseInsensitiveCompare(FLHyperKycConstants.LAUNCH) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return
                }
                let hyperKycConfigArgument = args[FLHyperKycConstants.HYPERKYCCONFIG] as! [String : Any]
                let completionHandler: (_ hyperKycResult: HyperKycResult) -> Void = {hyperKycResult in
                    
                    //  MARK: Process HyperKycResult and obtain final resultMap
                    let resultMap = processHyperKycResult(hyperKycResult: hyperKycResult)
                    
                    //  MARK: Pass the final resultMap to Flutter plugin
                    result(resultMap)
                }
                
                //  MARK: Call launch() function
                if let topController = UIApplication.shared.keyWindow?.rootViewController{
                    FLHyperKyc.launch(context: topController, hyperKycConfigArgument: hyperKycConfigArgument, callback: completionHandler)
                }
            } else {
                result(FlutterMethodNotImplemented)
            }
        })
    }
}

func processHyperKycResult(hyperKycResult: HyperKycResult) -> [String: Any] {
    
    //  MARK: Add result, kycData, errorMessage
    let resultEnum = hyperKycResult.result! as KYCResult
    var resultEnumString = ""
    var kycData = [
        FLHyperKycConstants.SELECTED_COUNTRY : "",
        FLHyperKycConstants.SELECTED_DOCUMENT : "",
        //  MARK: Added these when required (in the next steps)
        //  "docDataList" : [[String: Any]()],
        //  "faceData" : [String: Any](),
        //  "faceMatchData" : [String: Any](),
    ] as [String: Any]
    var errorMessage = ""
    
    //  MARK: Add resultEnum
    resultEnumString = String(describing: resultEnum)
    
    //  MARK: Add kycData
    if let hyperKycData = hyperKycResult.hyperKYCData {
        
        if let country = hyperKycData.selectedCountry {
            kycData[FLHyperKycConstants.SELECTED_COUNTRY] = country
        }
        if let document = hyperKycData.selectedDocument {
            kycData[FLHyperKycConstants.SELECTED_DOCUMENT] = document
        }
        
        //  MARK: Add docData to kycData
        var docDataList: [[String: Any]] = []
        for docData in hyperKycData.docDataList {
            var docDataDictionary = [
                FLHyperKycConstants.SIDE : FLHyperKycHelper.getStringFromProperty(property: docData.side),
                FLHyperKycConstants.DOC_IMAGE_PATH : FLHyperKycHelper.getStringFromProperty(property: docData.docImagePath),
                FLHyperKycConstants.ACTION : FLHyperKycHelper.getStringFromProperty(property: docData.action),
            ]
            if let responseHeaders = docData.responseHeaders  {
                docDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycHelper.getResponseHeadersJsonString(responseHeaders: responseHeaders)
            } else {
                docDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycConstants.EMPTY_JSON_STRING
            }
            if let responseResult = docData.responseResult {
                docDataDictionary[FLHyperKycConstants.RESPONSE_RESULT] = FLHyperKycHelper.getResponseResultJsonString(docDataResponseResult: responseResult, faceDataResponseResult: nil, faceMatchDataResponseResult: nil)
            } else {
                docDataDictionary[FLHyperKycConstants.RESPONSE_RESULT] = FLHyperKycConstants.EMPTY_JSON_STRING
            }
            docDataList.append(docDataDictionary as [String : Any])
        }
        kycData[FLHyperKycConstants.DOC_DATA_LIST] = docDataList
        
        //  MARK: Add faceData to kycData
        if let faceData = hyperKycData.faceData {
            var faceDataDictionary = [
                FLHyperKycConstants.CROPPED_FACE_IMAGE_PATH : FLHyperKycHelper.getStringFromProperty(property: faceData.croppedFaceImagePath),
                FLHyperKycConstants.FULL_FACE_IMAGE_PATH : FLHyperKycHelper.getStringFromProperty(property: faceData.fullFaceImagePath),
                FLHyperKycConstants.VIDEO_PATH : FLHyperKycHelper.getStringFromProperty(property: faceData.videoPath),
                FLHyperKycConstants.ACTION : FLHyperKycHelper.getStringFromProperty(property: faceData.action),
            ]
            if let responseHeaders = faceData.responseHeaders  {
                faceDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycHelper.getResponseHeadersJsonString(responseHeaders: responseHeaders)
            } else {
                faceDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycConstants.EMPTY_JSON_STRING
            }
            if let responseResult = faceData.responseResult {
                faceDataDictionary[FLHyperKycConstants.RESPONSE_RESULT] = FLHyperKycHelper.getResponseResultJsonString(docDataResponseResult: nil, faceDataResponseResult: responseResult, faceMatchDataResponseResult: nil)
            } else {
                faceDataDictionary[FLHyperKycConstants.RESPONSE_RESULT] = FLHyperKycConstants.EMPTY_JSON_STRING
            }
            kycData[FLHyperKycConstants.FACE_DATA] = faceDataDictionary
        } else {
            kycData[FLHyperKycConstants.FACE_DATA] = [String: Any]()
        }
        
        //  MARK: Add faceMatchData to kycData
        if let faceMatchData = hyperKycData.faceMatchData {
            var faceMatchDataDictionary = [
                FLHyperKycConstants.ACTION : FLHyperKycHelper.getStringFromProperty(property: faceMatchData.action),
            ]
            if let responseHeaders = faceMatchData.responseHeaders  {
                faceMatchDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycHelper.getResponseHeadersJsonString(responseHeaders: responseHeaders)
            } else {
                faceMatchDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycConstants.EMPTY_JSON_STRING
            }
            if let responseResult = faceMatchData.responseResult {
                faceMatchDataDictionary[FLHyperKycConstants.RESPONSE_RESULT] = FLHyperKycHelper.getResponseResultJsonString(docDataResponseResult: nil, faceDataResponseResult: nil, faceMatchDataResponseResult: responseResult)
            } else {
                faceMatchDataDictionary[FLHyperKycConstants.RESPONSE_RESULT] = FLHyperKycConstants.EMPTY_JSON_STRING
            }
            kycData[FLHyperKycConstants.FACE_MATCH_DATA] = faceMatchDataDictionary
        } else {
            kycData[FLHyperKycConstants.FACE_MATCH_DATA] = [String: Any]()
        }
    } else {
        kycData[FLHyperKycConstants.DOC_DATA_LIST] = []
        kycData[FLHyperKycConstants.FACE_DATA] = [String: Any]()
        kycData[FLHyperKycConstants.FACE_MATCH_DATA] = [String: Any]()
    }
    
    //  MARK: Add errorMessage
    if let errorMsg = hyperKycResult.errorMessage {
        errorMessage = errorMsg
    }
    
    //  MARK: Create the final result map which will include resultEnum, kycData and errorMessage
    let resultMap = [
        FLHyperKycConstants.RESULT : resultEnumString,
        FLHyperKycConstants.HYPERKYC_DATA : kycData,
        FLHyperKycConstants.ERROR_MESSAGE : errorMessage,
    ] as [String : Any]
    
    return resultMap
}
