import 'package:flutter/material.dart';

/// BaseResponse is not used anywhere currently
class BaseResponse {
  final Metadata? metadata;
  final String? status;
  final String? statusCode;

  /// Constructor
  BaseResponse({
    this.metadata,
    this.status,
    this.statusCode,
  });

  /// Convert BaseResponse object to Map
  Map<String, dynamic> toMap() {
    return {
      'metadata': this.metadata,
      'status': this.status,
      'statusCode': this.statusCode,
    };
  }

  /// Convert Map to BaseResponse object
  factory BaseResponse.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return BaseResponse();
    }
    return BaseResponse(
      metadata: map['metadata'] as Metadata,
      status: map['status'] as String,
      statusCode: map['statusCode'] as String,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'BaseResponse{metadata: $metadata, status: $status, statusCode: $statusCode}';
  }
}

/// Metadata holds the metadata received/present in Hyperverge API calls
class Metadata {
  final String? data;
  final String? customerId;
  final String? requestId;
  final String? transactionId;
  final String? attemptCount;

  /// Constructor
  Metadata({
    this.data,
    this.customerId,
    this.requestId,
    this.transactionId,
    this.attemptCount,
  });

  /// Convert Metadata object to Map
  Map<String, dynamic> toMap() {
    return {
      'data': this.data,
      'customerId': this.customerId,
      'requestId': this.requestId,
      'transactionId': this.transactionId,
      'attemptCount': this.attemptCount,
    };
  }

  /// Convert Map to Metadata object
  factory Metadata.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside Metadata.fromMap');

    if (map.isEmpty) {
      return Metadata();
    }
    return Metadata(
      data: map['data'] as String?,
      customerId: map['customerId'] as String?,
      requestId: map['requestId'] as String?,
      transactionId: map['transactionId'] as String?,
      attemptCount: map['attemptCount'] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'Metadata{data: $data, customerId: $customerId, requestId: $requestId, transactionId: $transactionId, attemptCount: $attemptCount}';
  }
}

/// Summary holds summary received from Hyperverge API calls
class Summary {
  final String? action;
  final List<Details?>? details;

  /// Constructor
  Summary({
    this.action,
    this.details,
  });

  /// Convert Summary object to Map
  Map<String, dynamic> toMap() {
    return {
      'action': this.action,
      'details': this.details,
    };
  }

  /// Convert Map to Summary object
  factory Summary.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside Summary.fromMap');

    if (map.isEmpty) {
      return Summary();
    }
    List<Details> detailsList = List.empty(growable: true);
    List detailsListFromMap = map['details'] ?? {};
    for (var detailMap in detailsListFromMap) {
      Details details = Details.fromMap(detailMap ?? {});
      detailsList.add(details);
    }
    return Summary(
      action: map['action'] as String?,
      details: detailsList,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'Summary{action: $action, details: $details}';
  }
}

/// Details holds the details present in Summary object
class Details {
  final String? code;
  final String? message;

  /// Constructor
  Details({
    this.code,
    this.message,
  });

  /// Convert Details object to Map
  Map<String, dynamic> toMap() {
    return {
      'code': this.code,
      'message': this.message,
    };
  }

  /// Convert Map to Details object
  factory Details.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside Details.fromMap');

    if (map.isEmpty) {
      return Details();
    }
    return Details(
      code: map['code'] as String?,
      message: map['message'] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'Details{code: $code, message: $message}';
  }
}

/// Fieldvalue is a generic object which holds the following properties obtained from Hyperverge API calls
class Fieldvalue {
  final String? value;
  final String? confidence;
  final String? score;
  final String? reviewNeeded;

  /// Constructor
  Fieldvalue({
    this.value,
    this.confidence,
    this.score,
    this.reviewNeeded,
  });

  /// Convert Fieldvalue object to Map
  Map<String, dynamic> toMap() {
    return {
      'value': this.value,
      'confidence': this.confidence,
      'score': this.score,
      'reviewNeeded': this.reviewNeeded,
    };
  }

  /// Convert map to Fieldvalue object
  factory Fieldvalue.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside Fieldvalue.fromMap');

    if (map.isEmpty) {
      return Fieldvalue();
    }
    return Fieldvalue(
      value: map['value'] as String?,
      confidence: map['confidence'] as String?,
      score: map['score'] as String?,
      reviewNeeded: map['reviewNeeded'] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'Fieldvalue{value: $value, confidence: $confidence, score: $score, reviewNeeded: $reviewNeeded}';
  }
}
