import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:hyperkyc_flutter/hyperkyc_api_base_response.dart';

/// FaceData contains face capture and liveness result
class FaceData {
  final String? croppedFaceImagePath;
  final String? fullFaceImagePath;
  final String? videoPath;
  final String? action;
  final String? responseHeaders;
  final FaceCaptureApiDetail? responseResult;

  /// Constructor
  FaceData({
    this.croppedFaceImagePath,
    this.fullFaceImagePath,
    this.videoPath,
    this.action,
    this.responseHeaders,
    this.responseResult,
  });

  /// Convert FaceData object to Map
  Map<String, dynamic> toMap() {
    return {
      'croppedFaceImagePath': this.croppedFaceImagePath,
      'fullFaceImagePath': this.fullFaceImagePath,
      'videoPath': this.videoPath,
      'action': this.action,
      'responseHeaders': this.responseHeaders,
      'responseResult': this.responseResult,
    };
  }

  /// Convert Map to FaceData object
  factory FaceData.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside FaceData.fromMap');
    if (map.isEmpty) {
      return FaceData();
    }
    return FaceData(
      croppedFaceImagePath: map['croppedFaceImagePath'] as String?,
      fullFaceImagePath: map['fullFaceImagePath'] as String?,
      videoPath: map['videoPath'] as String?,
      action: map['action'] as String?,
      responseHeaders: map['responseHeaders'] as String?,
      responseResult:
          FaceCaptureApiDetail.fromMap(jsonDecode(map['responseResult']) ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceData{croppedFaceImagePath: $croppedFaceImagePath, fullFaceImagePath: $fullFaceImagePath, videoPath: $videoPath, action: $action, responseHeaders: $responseHeaders, responseResult: $responseResult}';
  }
}

/// FaceCaptureApiDetail is response body from the Hyperverge liveness API call
class FaceCaptureApiDetail {
  final Metadata? metadata;
  final String? status;
  final String? statusCode;
  final FaceResultObject? result;

  /// Constructor
  FaceCaptureApiDetail({
    this.metadata,
    this.status,
    this.statusCode,
    this.result,
  });

  /// Convert FaceCaptureApiDetail object to Map
  Map<String, dynamic> toMap() {
    return {
      'metadata': this.metadata,
      'status': this.status,
      'statusCode': this.statusCode,
      'result': this.result,
    };
  }

  /// Convert Map to FaceCaptureApiDetail object
  factory FaceCaptureApiDetail.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside FaceCaptureApiDetail.fromMap');
    if (map.isEmpty) {
      return FaceCaptureApiDetail();
    }
    return FaceCaptureApiDetail(
      metadata: Metadata.fromMap(map['metadata'] ?? {}),
      status: map['status'] as String?,
      statusCode: map['statusCode'] as String?,
      result: FaceResultObject.fromMap(map['result'] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceCaptureApiDetail{metadata: $metadata, status: $status, statusCode: $statusCode, result: $result}';
  }
}

/// FaceResultObject
class FaceResultObject {
  final LivenessApiStructure? details;
  final String? error;
  final Summary? summary;

  /// Constructor
  FaceResultObject({
    this.details,
    this.error,
    this.summary,
  });

  /// Convert FaceResultObject object to Map
  Map<String, dynamic> toMap() {
    return {
      'details': this.details,
      'error': this.error,
      'summary': this.summary,
    };
  }

  /// Convert Map to FaceResultObject object
  factory FaceResultObject.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside FaceResultObject.fromMap');
    if (map.isEmpty) {
      return FaceResultObject();
    }
    return FaceResultObject(
      details: LivenessApiStructure.fromMap(map['details'] ?? {}),
      error: map['error'] as String?,
      summary: Summary.fromMap(map['summary'] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceResultObject{details: $details, error: $error, summary: $summary}';
  }
}

/// LivenessApiStructure
class LivenessApiStructure {
  final Fieldvalue? liveFace;
  final String? live;
  final String? livenessScore;
  final String? toBeReviewed;
  final FaceQualityChecks? qualityChecks;
  final String? error;

  /// Constructor
  LivenessApiStructure({
    this.liveFace,
    this.live,
    this.livenessScore,
    this.toBeReviewed,
    this.qualityChecks,
    this.error,
  });

  /// Convert LivenessApiStructure object to Map
  Map<String, dynamic> toMap() {
    return {
      'liveFace': this.liveFace,
      'live': this.live,
      'livenessScore': this.livenessScore,
      'toBeReviewed': this.toBeReviewed,
      'qualityChecks': this.qualityChecks,
      'error': this.error,
    };
  }

  /// Convert Map to LivenessApiStructure object
  factory LivenessApiStructure.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside LivenessApiStructure.fromMap');
    if (map.isEmpty) {
      return LivenessApiStructure();
    }
    return LivenessApiStructure(
      liveFace: Fieldvalue.fromMap(map['liveFace'] ?? {}),
      live: map['live'] as String?,
      livenessScore: map['livenessScore'] as String?,
      toBeReviewed: map['toBeReviewed'] as String?,
      qualityChecks: FaceQualityChecks.fromMap(map['qualityChecks'] ?? {}),
      error: map['error'] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'LivenessApiStructure{liveFace: $liveFace, live: $live, livenessScore: $livenessScore, toBeReviewed: $toBeReviewed, qualityChecks: $qualityChecks, error: $error}';
  }
}

/// FaceQualityChecks
class FaceQualityChecks {
  final Fieldvalue? eyesClosed;
  final Fieldvalue? maskPresent;
  final Fieldvalue? multipleFaces;
  final Fieldvalue? blur;

  /// Constructor
  FaceQualityChecks({
    this.eyesClosed,
    this.maskPresent,
    this.multipleFaces,
    this.blur,
  });

  /// Convert FaceQualityChecks object to Map
  Map<String, dynamic> toMap() {
    return {
      'eyesClosed': this.eyesClosed,
      'maskPresent': this.maskPresent,
      'multipleFaces': this.multipleFaces,
      'blur': this.blur,
    };
  }

  /// Convert Map to FaceQualityChecks object
  factory FaceQualityChecks.fromMap(Map<dynamic, dynamic> map) {
    debugPrint('inside FaceQualityChecks.fromMap');

    if (map.isEmpty) {
      return FaceQualityChecks();
    }
    return FaceQualityChecks(
      eyesClosed: Fieldvalue.fromMap(map['eyesClosed'] ?? {}),
      maskPresent: Fieldvalue.fromMap(map['maskPresent'] ?? {}),
      multipleFaces: Fieldvalue.fromMap(map['multipleFaces'] ?? {}),
      blur: Fieldvalue.fromMap(map['blur'] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'FaceQualityChecks{eyesClosed: $eyesClosed, maskPresent: $maskPresent, multipleFaces: $multipleFaces, blur: $blur}';
  }
}
