import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:hyperkyc_flutter/hyperkyc_config.dart';
import 'package:hyperkyc_flutter/hyperkyc_flutter_constants.dart';
import 'package:hyperkyc_flutter/hyperkyc_result.dart';

class HyperKyc {
  static const MethodChannel _channel =
      MethodChannel(HyperKycConstants.hyperKycChannel);

  /// Launch HyperKyc SDK using `HyperkycConfig` instance
  /// Returns `hyperKycResult` (a Map which consists of following 3 objects)
  ///  {
  ///     'result' : 'cancelled' or 'failure' or 'success',
  ///	    'hyperkycData' : hyperKycData,
  ///	    'errorMessage' : empty or 'failureErrorMessage'
  ///  }
  ///  Please contact Hyperverge team to get access for complete documentation on how to process hyperKycResult
  static Future<HyperKycResult> launch(
      {required HyperKycConfig hyperKycConfig}) async {
    Map hyperKycResultMap = {};
    HyperKycResult hyperKycResult = HyperKycResult();
    try {
      hyperKycResultMap = await _channel.invokeMethod(
        HyperKycConstants.launch,
        {
          'hyperKycConfig': hyperKycConfig.toMap(),
        },
      );
      hyperKycResult = HyperKycResult.fromMap(hyperKycResultMap);
      debugPrint('hyperKycResult Object : ${hyperKycResult.toString()}');
    } catch (e) {
      print(e.toString());
    }
    return hyperKycResult;
  }
}
