import 'hyperkyc_data.dart';

/// HyperKycResult enum will state whether the hyperkyc flow was success or cancelled or failure.
enum HyperKycResultEnum {
  success,
  failure,
  cancelled,
}

/// HyperKycResult contains the result from the HyperKyc workflow
class HyperKycResult {
  final HyperKycResultEnum? result;
  final HyperKycData? hyperKYCData;
  final String? errorMessage;

  /// Constructor
  HyperKycResult({
    this.result,
    this.hyperKYCData,
    this.errorMessage,
  });

  /// Convert HyperKycResult object to Map
  Map<String, dynamic> toMap() {
    return {
      'result': this.result?.name,
      'hyperKYCData': this.hyperKYCData,
      'errorMessage': this.errorMessage,
    };
  }

  /// Convert Map to HyperKycResult object
  factory HyperKycResult.fromMap(Map<dynamic, dynamic> map) {
    HyperKycResultEnum hyperKycResultEnum = HyperKycResultEnum.failure;
    if (map['result'] != null) {
      String resultEnumString = map['result'];
      switch (resultEnumString) {
        case 'success':
          hyperKycResultEnum = HyperKycResultEnum.success;
          break;
        case 'cancelled':
          hyperKycResultEnum = HyperKycResultEnum.cancelled;
          break;
        case 'failure':
          hyperKycResultEnum = HyperKycResultEnum.failure;
          break;
      }
    }
    return HyperKycResult(
      result: hyperKycResultEnum,
      hyperKYCData: HyperKycData.fromMap(map['hyperKYCData'] ?? {}),
      errorMessage: map['errorMessage'] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'HyperKycResult{result: $result, hyperKYCData: $hyperKYCData, errorMessage: $errorMessage}';
  }
}
